/**
 * Copyright (C) 2010 Cloudfarming <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.planning.tactical;

import java.util.Date;
import java.util.List;
import nl.cloudfarming.client.model.CropProtection;
import nl.cloudfarming.client.model.CropProtectionTask;
import nl.cloudfarming.client.model.DataProvider;
import nl.cloudfarming.client.model.Field;
import nl.cloudfarming.client.model.FieldPlan;
import nl.cloudfarming.client.model.Operation;
import nl.cloudfarming.client.model.PartField;
import nl.cloudfarming.client.model.ProductionUnit;
import nl.cloudfarming.client.model.Task;
import org.openide.nodes.Node;

/**
 * Interface for service classes in planning-tactical.
 * This is done so that all implementations can be final and the services are still testable
 * @author merijn
 */
public interface Service extends DataProvider{
    /**
     * Updates a Operation. 
     * @param toBeMerged
     * @throws IllegalArgumentException: when toBemerged is not persisted yet.
     */
    void updateOperation(Operation toBeMerged);

    /**
     * get all the fieldPlan nodes. The children of fieldPlan
     * @return Node[] FieldPlanNodes 
     */
    public Node[] getFieldplanNodes();

    /**
     * Create a PartField from a Field and a selected ProductionUnit
     * 
     * The new PartField will inherit start- and enddate from ProductionUnit and Acreage and WKT from the Field
     */
    public PartField createPartField(Field field, ProductionUnit productionUnit);

    /**
     * updates a task
     * @param toBeMerged
     * @throws IllegalArgumentException: when toBemerged is not persisted yet.
     */
    public void updateTask(Task toBeMerged);

    /**
     * updates a productionUnit
     * @param toBeMerged
     * @throws IllegalArgumentException: when toBemerged is not persisted yet.
     */
    public void updateProductionUnit(ProductionUnit toBeMerged);

    /**
     * Creates an Operation within a transaction. Starts a transaction if it is not already started.
     * @param operation used as parameter placeholder, use return value
     * @return 
     */
    public Operation createOperation(Operation.Builder operationBuilder);

    /**
     * Creates an Operation within a transaction. Starts a transaction if it is not already started.
     * @param operation used as parameter
     * @return persisted Operation
     */
    public Operation createOperation(Operation operation);

    /**
     * Creates an Task within a transaction. Starts a transaction if it is not already started.
     * @param Task used as parameter placeholder, use return value
     * @return 
     */
    public Task createTask(Task.Builder taskBuilder);

    /**
     * Creates a productionUnit within a transaction. Starts a transaction if it is not already started.
     * @param productionUnitBuilder PUBuilder to build productionUnit from.
     * @return 
     */
    public ProductionUnit createProductionUnit(ProductionUnit.Builder productionUnitBuilder);

    /**
     * Creates a CropProtection within a transaction. Starts a transaction if it is not already started.
     * @param CropProtection used as parameter placeholder, use return value
     * @return 
     */
    public CropProtection createCropProtection(CropProtection.Builder cropProtectionBuilder);

    /**
     * Creates a CropProtectionTask within a transaction. Starts a transaction if it is not already started.
     * @param CropProtectionTask used as parameter placeholder, use return value
     * @return 
     */
    public CropProtectionTask createCropProtectionTask(CropProtectionTask.Builder cropProtectionTaskBuilder);

    /**
     * Creates a CropProtectionTask within a transaction. Starts a transaction if it is not already started.
     * @param CropProtectionTask used as parameter placeholder, use return value
     * @return 
     */
    public CropProtectionTask createCropProtectionTask(CropProtectionTask cropProtectionTask);

    /**
     * Creates an FieldPlan within a transaction. Starts a transaction if it is not already started.
     * @param year for which Fieldplan is created
     */
    public FieldPlan createFieldPlanForYear(String name, int year);

    /**
     * Todo: Merged in from base. Is this still being used?
     * find all fieldplans
     *
     * @return List of FieldPlans
     */
    public List<FieldPlan> findFieldPlans();

    /**
     * Finds all available productionUnits
     * @return List of productionUnits
     */
    public List<ProductionUnit> findAllProductionUnits();

    /**
     * Finds al the available Operations
     * @return List of Operation
     */
    public List<Operation> findAllOperations();
    
    
    /**
     * find all fieldplans
     *
     * @return List of FieldPlans
     */
    public List<FieldPlan> findAllFieldPlans();

    /**
     * Find all the available Tasks
     * @return List Task
     */
    public List<Task> findAllTasks();

    /**
     * Find all the available Fields
     * @return List Field
     */
    public List<PartField> findAllPartFields();

    /**
     * Finds all the operations which are connected to the productionUnit via task
     * @param id the id of the productionUnit
     * @return List List of operations linked to the specified productionUnit
     */
    public List<Task> findTaskByProductionUnitId(Long id);

    /**
     * Removes productionUnit from fieldplan. Checks if the productionUnit which will be removed is managed by the enttity manager
     * @param productionUnit which will be removed.
     */
    public void removeProductionUnitFromFieldPlan(ProductionUnit productionUnit);

    /**
     * Find Productionunits where Date date is between startDate and endDate
     * @param date The date which must be between startDate and endDate
     * @return List<ProductionUnit> the ProductionUnits which match.
     */
    public List<ProductionUnit> findProductionUnitsByDate(Date date);

    /**
     * Find all the CropProtections
     * @return List<CropProtection> all the available CropProtections
     */
    public List<CropProtection> findAllCropProtections();

    /**
     * Finds All the cropProtections based on there name and there lastUsage date
     * @param name the name of the cropProtection
     * @param lastUsage the LastUsage Date
     * @return the CropProtection found.
     */
    public CropProtection findCropProtectionByNameAndLastUsage(String name, Date lastUsage);

    /**
     * Finds all the cropProtectiontasks that are available
     * @return a list of the CropProtectionTasks found
     */
    public List<CropProtectionTask> findAllCropProtectionTasks();

    /*
     * find all productionUnits by FieldPlan 
     *
     * @return List of FieldPlans
     */
    public List<ProductionUnit> findProductionUnitsByFieldPlan(FieldPlan fieldPlan);

    
}
